#!/bin/bash
########################################
#
# @file   install.sh
# @brief  Installer file for an easy Raspberry Pi or VM implementation
#
# multiOTP package - Strong two-factor authentication open source package
# http://www.multiOTP.net/
#
# The multiOTP package is the lightest package available that provides so many
# strong authentication functionalities and goodies, and best of all, for anyone
# that is interested about security issues, it's a fully open source solution!
#
# This package is the result of a *LOT* of work. If you are happy using this
# package, [Donation] are always welcome to support this project.
# Please check http://www.multiOTP.net/ and you will find the magic button ;-)
#
# @author    Andre Liechti, SysCo systemes de communication sa, <developer@sysco.ch>
# @version   4.3.4.2-beta-0
# @date      2016-05-21
# @since     2013-11-29
# @copyright (c) 2013-2016 by SysCo systemes de communication sa
# @copyright GNU Lesser General Public License
#
########################################

FAMILY=""
UNAME=$(uname -a)
if [[ "${UNAME}" == *armv8* ]]; then
    HARDWARE=$(cat /proc/cpuinfo | grep "Hardware" | awk -F': ' '{print $2}')
    if [[ "${HARDWARE}" == *BCM27* ]]; then
        # Raspberry Pi 3 B
        FAMILY="RASPBERRY"
        TYPE="RP3"
    else
        # Nothing else yet !
        FAMILY="RASPBERRY"
        TYPE="RP3"
    fi
elif [[ "${UNAME}" == *armv7l* ]]; then
    HARDWARE=$(cat /proc/cpuinfo | grep "Hardware" | awk -F': ' '{print $2}')
    if [[ "${HARDWARE}" == *BCM27* ]]; then
        LSCPU=$(/usr/bin/lscpu | grep "CPU max MHz" | awk -F': ' '{print $2}')
        if [[ "${LSCPU}" == *1200* ]]; then
            # Raspberry Pi 3
            FAMILY="RASPBERRY"
            TYPE="RP3"
        else
            # Raspberry Pi 2
            FAMILY="RASPBERRY"
            TYPE="RP2"
        fi
    else
        # Beaglebone Black or similar
        FAMILY="ARM"
        TYPE="BBB"
    fi
elif [[ "${UNAME}" == *armv6l* ]]; then
    # Raspberry Pi B/B+
    FAMILY="RASPBERRY"
    TYPE="RPI"
else
    # others (Virtual Appliance)
    FAMILY="VAP"
    TYPE="VAP"
fi


# Remove X11 related packages as we don't need any X Window System
apt-get -y autoremove x11-common

# Remove ntpdate as ntp will do the job
apt-get -y autoremove ntpdate


# Purge unused packages
apt-get -y purge gpicview xarchiver xpdf galculator scratch dillo midori netsurf-gtk
apt-get -y purge omxplayer alsa-base alsa-utils penguinspuzzle python-pygame
apt-get -y purge lxde-core lxde-common lxde-icon-theme lxinput lxmenu-data lxpolkit
apt-get -y purge lxrandr lxsession lxsession-edit lxshortcut lxtask lxterminal
apt-get -y purge lxappearance lightdm lightdm-gtk-greeter openbox ttf-dejavu-core
apt-get -y purge fonts-freefont-ttf wpagui x11-common dbus-x11 desktop-base
apt-get -y purge desktop-file-utils hicolor-icon-theme leafpad xdg-utils xauth
apt-get -y purge gnome-themes-standard gtk2-engines raspberrypi-artwork
apt-get -y purge fontconfig fontconfig-config ttf-dejavu-core


# Cleaning dpkg
dpkg --configure -a


# apt-get update and upgrade
apt-get -y update
apt-get -y upgrade


# Prepare automatic answers for IP table
echo iptables-persistent iptables-persistent/autosave_v4 boolean false | debconf-set-selections
echo iptables-persistent iptables-persistent/autosave_v6 boolean false | debconf-set-selections


# apt-get additional packages installation
apt-get -y install apache2-utils
apt-get -y install dselect
apt-get -y install freeradius
apt-get -y install iptables-persistent
apt-get -y install nginx-extras
apt-get -y install ntp
apt-get -y install php5-fpm php5-cgi php5-gd php5-mcrypt php5-gmp php5-sqlite
apt-get -y install php5-dev bzip2 libbz2-dev php-pear build-essential
apt-get -y install php5-ldap
apt-get -y install sqlite
apt-get -y install subversion
apt-get -y install sudo

apt-get -y install php-apc


# Install some additional packages on Virtual Appliance
if [[ "${TYPE}" == "VAP" ]]; then
	# Prepare automatic answers for MySQL
	echo mysql-server-5.1 mysql-server/root_password password fJKGeztDF3456DvB | debconf-set-selections
	echo mysql-server-5.1 mysql-server/root_password_again password fJKGeztDF3456DvB | debconf-set-selections

	apt-get -y install mysql-server
	apt-get -y install mysql-common
    apt-get -y install php5-mysql
else
    apt-get -y install fake-hwclock
    apt-get -y install i2c-tools
fi


# Clean unused packages
apt-get -y autoremove


# More cleaning
apt-get autoclean
# apt-get -y install localepurge


# Copy the preconfigured files for multiOTP
cp -f -R /boot/multiotp-tree/* /
rm -f -R /boot/multiotp-tree


if [[ "${TYPE}" == "VAP" ]]; then
    # No script for the hardware clock
    rm -f /etc/init.d/multiotp
    
    # No config file for Virtual Appliance
    rm -f /boot/config.txt
    
    # IP address definition
    echo auto lo > /etc/network/interfaces
    echo  >> /etc/network/interfaces
    echo iface lo inet loopback >> /etc/network/interfaces
    echo allow-hotplug eth0 >> /etc/network/interfaces
    echo iface eth0 inet static >> /etc/network/interfaces
    echo address 192.168.1.44 >> /etc/network/interfaces
    echo netmask 255.255.255.0 >> /etc/network/interfaces
    echo network 192.168.1.0 >> /etc/network/interfaces
    echo gateway 192.168.1.1 >> /etc/network/interfaces
fi

# Create multiOTP folders
mkdir /etc/multiotp
mkdir /etc/multiotp/config
mkdir /etc/multiotp/devices
mkdir /etc/multiotp/groups
mkdir /etc/multiotp/tokens
mkdir /etc/multiotp/users
mkdir /var/log/multiotp


# Create an SSL certificate
mkdir /etc/multiotp/certificates
rm -f /etc/multiotp/certificates/*
openssl genrsa -out /etc/multiotp/certificates/multiotp.key 2048
openssl req -new -key /etc/multiotp/certificates/multiotp.key -out /etc/multiotp/certificates/multiotp.csr -subj "/C=CH/ST=GPL/L=Open Source Edition/O=multiOTP/OU=strong authentication server/CN=multiOTP"
openssl x509 -req -days 3652 -in /etc/multiotp/certificates/multiotp.csr -signkey /etc/multiotp/certificates/multiotp.key -out /etc/multiotp/certificates/multiotp.crt


# Change various rights
chmod g+w    /etc/freeradius/clients.conf
chmod g+w    /etc/freeradius/users

if [[ ! "${TYPE}" == "VAP" ]]; then
    chmod 755    /etc/init.d/multiotp
fi

chmod 777 -R /etc/multiotp
chmod 777 -R /usr/local/bin/multiotp
chmod 777 -R /var/log/multiotp


# Change some owners
chown -R www-data:www-data /etc/multiotp
chown -R www-data:www-data /usr/local/bin/multiotp
chown -R www-data:www-data /var/log/multiotp


# Adapting nginx configuration
rm -f /etc/nginx/sites-enabled/default
ln -s /etc/nginx/sites-available/multiotp /etc/nginx/sites-enabled/multiotp
ln -s /etc/nginx/sites-available/multiotp-proxy /etc/nginx/sites-enabled/multiotp-proxy
sed -i 's/# multi_accept on;/multi_accept on;/' /etc/nginx/nginx.conf
# SSLv3 protocol removed due to the POODLE attack (CVE-2014-3566)
sed -i 's/SSLv3 //' /etc/nginx/sites-available/multiotp
/etc/init.d/nginx restart


# Adapting php-apc configuration
sed -i '/^apc.shm_size=/d' /etc/php5/conf.d/20-apc.ini
sed -i '/^apc.max_file_size=/d' /etc/php5/conf.d/20-apc.ini
sed -i '1i apc.shm_size=48M' /etc/php5/conf.d/20-apc.ini
sed -i '1i apc.max_file_size=48M' /etc/php5/conf.d/20-apc.ini
/etc/init.d/php5-fpm restart


# Timezone definition
echo "Europe/Zurich" > /etc/timezone
rm -f /etc/localtime && cp -f /usr/share/zoneinfo/Europe/Zurich /etc/localtime


# Stop NTP, update date/time, restart NTP and check configuration
/etc/init.d/ntp stop
/etc/init.d/ntp restart
ntpq -p
date -R


if [[ ! "${TYPE}" == "VAP" ]]; then

    # Install Afterthought Software RasClock (and other PCF212x compatible RTC clock) driver and load drivers at boot

    cd /tmp

    # Package version 3.2.27
    # wget http://afterthoughtsoftware.com/files/linux-image-3.2.27-atsw-rtc_1.0_armhf.deb
    # dpkg -i linux-image-3.2.27-atsw-rtc_1.0_armhf.deb
    # cp /boot/vmlinuz-3.2.27-atsw-rtc+ /boot/kernel.img

    # Package version 3.6.11
    wget http://afterthoughtsoftware.com/files/linux-image-3.6.11-atsw-rtc_1.1_armhf.deb
    dpkg -i linux-image-3.6.11-atsw-rtc_1.1_armhf.deb
    cp /boot/vmlinuz-3.6.11-atsw-rtc+ /boot/kernel.img


    # Enable I2C support
    sed -i 's/^blacklist i2c-bcm2708/# blacklist i2c-bcm2708/' /etc/modprobe.d/raspi-blacklist.conf

    /bin/grep "i2c-dev" /etc/modules > /dev/null 2>&1
    if [ $? != 0 ]; then
        echo i2c-dev >> /etc/modules
    fi
    /bin/grep "i2c-bcm2708" /etc/modules > /dev/null 2>&1
    if [ $? != 0 ]; then
        echo i2c-bcm2708 >> /etc/modules
    fi

    
    # Enable Afterthought Software RasClock module (and other PCF212x compatible RTC clock)
    /bin/grep "rtc-pcf2127a" /etc/modules > /dev/null 2>&1
    if [ $? != 0 ]; then
        echo rtc-pcf2127a >> /etc/modules
    fi

    
    # Enable CJE Micros RTC clock module (and other DSxxxx compatible RTC clock)
    /bin/grep "rtc-ds1307" /etc/modules > /dev/null 2>&1
    if [ $? != 0 ]; then
        echo rtc-ds1307 >> /etc/modules
    fi

    # Install multiotp service (only for Virtual Appliance)
    insserv multiotp

fi




# Check and add freeradius config if needed
/bin/grep "multiotp my-first-network" /etc/freeradius/clients.conf > /dev/null 2>&1
if [ $? != 0 ]; then
    echo "# multiotp my-first-network BEGIN" >> /etc/freeradius/clients.conf
    echo "client 192.168.0.0/16 { # multiotp my-first-network" >> /etc/freeradius/clients.conf
    echo "secret = myfirstpass # multiotp my-first-network" >> /etc/freeradius/clients.conf
    echo "shortname = my-first-network # multiotp my-first-network" >> /etc/freeradius/clients.conf
    echo "} # multiotp my-first-network" >> /etc/freeradius/clients.conf
    echo "# multiotp my-first-network END" >> /etc/freeradius/clients.conf
fi


# Add some users in groups (usermod -a -G groupName userName)
usermod -a -G www-data freerad
usermod -a -G freerad www-data


# Restart freeradius service
/etc/init.d/freeradius restart


#iptable

# autorizing PING
iptables -A INPUT -p icmp -j ACCEPT

# authorized ports
iptables -A INPUT -p tcp --dport 22 -j ACCEPT
iptables -A INPUT -p tcp --dport 80 -j ACCEPT
iptables -A INPUT -p tcp --dport 443 -j ACCEPT
iptables -A INPUT -p udp --dport 1812 -j ACCEPT
iptables -A INPUT -p udp --dport 1813 -j ACCEPT

# no firewall on the local loop (127.x.x.x)
iptables -A INPUT -i lo -j ACCEPT
iptables -A OUTPUT -o lo -j ACCEPT

# existing connections receive their traffic
iptables -A INPUT -m state --state ESTABLISHED,RELATED -j ACCEPT

# refused by default
iptables -P INPUT DROP

iptables-save > /etc/iptables/rules.v4

# ip6 can be swapped if not used
ip6tables-save > /etc/iptables/rules.v6


# Specific multiOTP configuration
/usr/local/bin/multiotp/multiotp.php -config debug=1


if [[ "${TYPE}" == "VAP" ]]; then
    /usr/bin/mysql -u root -pfJKGeztDF3456DvB -e "create database multiotp; GRANT ALL PRIVILEGES ON multiotp.* TO multiotp@localhost IDENTIFIED BY 'dfh45AReTZTxsdR';"
    /usr/local/bin/multiotp/multiotp.php -config sql-server=127.0.0.1 sql-username=multiotp sql-password=dfh45AReTZTxsdR sql-database=multiotp
    /usr/local/bin/multiotp/multiotp.php -config backend-type=mysql
    /usr/local/bin/multiotp/multiotp.php -initialize-backend
fi


# Remove speaker support
/bin/grep "pcspkr" /etc/modprobe.d/blacklist.conf > /dev/null 2>&1
if [ $? != 0 ]; then
    echo 'blacklist pcspkr' >> /etc/modprobe.d/blacklist.conf
    echo 'blacklist snd_pcsp' >> /etc/modprobe.d/blacklist.conf
fi


# Remove this file
rm -f /boot/install.sh


# Clean the history
history -c


#Clean all unnecessary files
rm -r /home/pi/Desktop /home/pi/python_games /home/pi/ocr_pi.png
# localepurge
apt-get clean


# And finally reboot the device if we are not updating only
if [[ ! "$1" == "update" ]]; then
    reboot
fi
